Object.defineProperty(exports, "__esModule", {
  value: true
});

var _templateObject = _taggedTemplateLiteral(['', ''], ['', '']);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _babylon = require('babylon');

var _graphqlTag = require('graphql-tag');

var _graphqlTag2 = _interopRequireDefault(_graphqlTag);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _taggedTemplateLiteral(strings, raw) { return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var resolve = void 0;

exports.default = function (_ref) {
  var t = _ref.types;
  return {
    manipulateOptions: function manipulateOptions(_ref2) {
      var resolveModuleSource = _ref2.resolveModuleSource;

      if (!resolve) {
        resolve = resolveModuleSource || function (src, file) {
          return _path2.default.resolve((0, _path.dirname)(file), src);
        };
      }
    },

    visitor: {
      ImportDeclaration: {
        exit: function exit(curPath, _ref3) {
          var babelPath = _ref3.file.opts.filename,
              nodePath = _ref3.opts.nodePath;

          var importPath = curPath.node.source.value;
          if (importPath.endsWith('.graphql') || importPath.endsWith('.gql')) {
            var fallbackResolvePaths = nodePath ? nodePath.split(_path2.default.delimiter) : undefined;
            var query = createQuery(importPath, babelPath, fallbackResolvePaths);
            query.processFragments();
            query.parse();
            query.dedupeFragments();
            query.makeSourceEnumerable();
            curPath.replaceWith(buildInlineVariableAST(query.ast));
          }

          function buildInlineVariableAST(graphqlAST) {
            var inlineVarName = curPath.node.specifiers[0].local.name;
            return (0, _babylon.parse)('const ' + inlineVarName + ' = ' + JSON.stringify(graphqlAST)).program.body[0];
          }
        }
      }
    }
  };
};

function createQuery(queryPath, babelPath, paths) {
  var absPath = resolve(queryPath, babelPath);
  if (!(0, _fs.existsSync)(absPath)) absPath = require.resolve(queryPath, { paths: paths });
  var source = (0, _fs.readFileSync)(absPath).toString();
  var ast = null;
  var fragmentDefs = [];

  return {
    processFragments: function processFragments() {
      processImports(getImportStatements(source), absPath);

      function getImportStatements(src) {
        return src.replace(/\r/g, '').split(/\n+/g).filter(function (line) {
          return line.startsWith('#import');
        });
      }

      function processImports(imports, relFile) {
        imports.forEach(function (statement) {
          var fragmentPath = statement.split(/[\s\n]+/g)[1].slice(1, -1);
          var absFragmentPath = resolve(fragmentPath, relFile);
          var fragmentSource = (0, _fs.readFileSync)(absFragmentPath.replace(/'/g, '')).toString();
          var subFragments = getImportStatements(fragmentSource);
          if (subFragments.length > 0) {
            processImports(subFragments, absFragmentPath);
          }
          // prettier-ignore
          fragmentDefs = [].concat(_toConsumableArray((0, _graphqlTag2.default)(_templateObject, fragmentSource).definitions), _toConsumableArray(fragmentDefs));
        });
      }
    },
    parse: function parse() {
      // prettier-ignore
      var parsedAST = (0, _graphqlTag2.default)(_templateObject, source);
      parsedAST.definitions = [].concat(_toConsumableArray(parsedAST.definitions), _toConsumableArray(fragmentDefs));
      ast = parsedAST;
    },
    dedupeFragments: function dedupeFragments() {
      var seenNames = {};
      ast.definitions = ast.definitions.filter(function (def) {
        if (def.kind !== 'FragmentDefinition') return true;
        return seenNames[def.name.value] ? false : seenNames[def.name.value] = true;
      });
    },
    makeSourceEnumerable: function makeSourceEnumerable() {
      var newAST = JSON.parse(JSON.stringify(ast));
      newAST.loc.source = ast.loc.source;
      ast = newAST;
    },

    get ast() {
      return ast;
    }
  };
}